"""Plugin for generic approval flows."""


from django.contrib.contenttypes.models import ContentType
from django.core.validators import MinValueValidator
from django.utils.translation import ugettext_lazy as _
from InvenTree.helpers_model import notify_responsible
from InvenTree.status_codes import PurchaseOrderStatus
from order.models import PurchaseOrder
from plugin import InvenTreePlugin
from plugin.events import trigger_event
from plugin.mixins import (
    AppMixin,
    NavigationMixin,
    SettingsContentMixin,
    SettingsMixin,
    UrlsMixin,
)
from users.models import Owner

from .notifications import ApprovalStarted


class ApprovalPlugin(
    AppMixin,
    SettingsMixin,
    UrlsMixin,
    NavigationMixin,
    SettingsContentMixin,
    InvenTreePlugin,
):
    """Plugin for generic approval flows."""

    NAME = "ApprovalPlugin"
    SLUG = "inventree_approval"
    TITLE = "Approvals"
    SETTINGS = {
        "APPROVAL_MINIMUM_APPROVERS": {
            "name": _("Minimum Approvers"),
            "description": _("Minimum number of approvers required for approval"),
            "default": 2,
            "validator": [int, MinValueValidator(1)],
        },
        "APPROVAL_MAXIMUM_DENIERS": {
            "name": _("Maximum Deniers"),
            "description": _("Maximum number of deniers required for denial"),
            "default": 1,
            "validator": [int, MinValueValidator(1)],
        },
    }
    NAVIGATION_TAB_NAME = "Approvals"
    NAVIGATION = [
        {"name": "Overview", "link": f"plugin:{SLUG}:approval-ui"},
    ]

    def get_settings_content(self, request):
        """Custom settings content for the plugin."""
        return """
        <h3>Warning</h3>
        <p>This plugin is currently in closed alpha and not open source. All rights are reserved by InvenHost and MjMair.<br>
        More information can be found on the <a href="https://github.com/invenhost/plugins/issues/1">public EPIC</a>.</p>
        """

    def setup_urls(self):
        """Setup url endpoints for this plugin."""
        from .api import urlpatterns

        return urlpatterns


# define transition method
try:
    from generic.states import TransitionMethod
except ImportError:

    class TransitionMethod:
        """Dummy class for transition method."""


contenttype_obj = ContentType.objects.get_for_model(PurchaseOrder)


class POchanges(TransitionMethod):
    """Transition method for PurchaseOrder objects."""

    def transition(current_state, target_state, instance, default_action, **kwargs):  # noqa: N805
        """Example override function for state transition."""
        from .models import Approval

        if not isinstance(instance, PurchaseOrder):
            return False

        # Original code from src/inventree/orders/models.py
        if instance.can_cancel:
            # is_active = getSetting("PURCHASEORDER_REQUIRE_APPROVAL")
            is_active = True

            if (
                is_active
                and Approval.objects.filter(
                    object_id=instance.id, content_type=contenttype_obj
                ).count()
                == 0
            ):
                if instance.responsible is None:
                    instance.responsible = Owner.create(obj=instance.created_by)

                Approval.objects.create(
                    name=f"PO: {instance.reference}",
                    content_object=instance,
                    created_by=instance.created_by,
                    modified_by=instance.created_by,
                    responsible=instance.responsible,
                )

                instance.status = PurchaseOrderStatus.PENDING_APPROVAL.value
                instance.save()

                trigger_event("purchaseorder.approval_start", id=instance.pk)

                # Notify users that an approval was started
                notify_responsible(
                    instance,
                    PurchaseOrder,
                    content=ApprovalStarted,
                )
                return instance
            else:
                # Run the default action
                return False
